@@ -236,13 +236,14 @@ const renderFilterLists = ( ) => {
             }
             const groupDetails = listTree[groupKey];
             if ( listDetails.parent !== undefined ) {
-                if ( groupDetails.lists[listDetails.parent] === undefined ) {
-                    groupDetails.lists[listDetails.parent] = {
-                        title: listDetails.parent,
-                        lists: {},
-                    };
+                let lists = groupDetails.lists;
+                for ( const parent of listDetails.parent.split('|') ) {
+                    if ( lists[parent] === undefined ) {
+                        lists[parent] = { title: parent, lists: {} };
+                    }
+                    lists = lists[parent].lists;
                 }
-                groupDetails.lists[listDetails.parent].lists[listkey] = listDetails;
+                lists[listkey] = listDetails;
             } else {
                 listDetails.title = listNameFromListKey(listkey);
                 groupDetails.lists[listkey] = listDetails;
@@ -399,9 +400,10 @@ const toggleFilterList = (elem, on, ui = false) => {
 const updateListNode = listNode => {
     if ( listNode === null ) { return; }
     if ( listNode.dataset.role !== 'node' ) { return; }
-    const listLeaves = qsa$(listNode, '.listEntry[data-role="leaf"].checked');
+    const checkedListLeaves = qsa$(listNode, '.listEntry[data-role="leaf"].checked');
+    const allListLeaves = qsa$(listNode, '.listEntry[data-role="leaf"]');
     dom.text(qs$(listNode, '.nodestats'),
-        renderNodeStats(listLeaves.length, qsa$(listNode, '.listEntry[data-role="leaf"]').length)
+        renderNodeStats(checkedListLeaves.length, allListLeaves.length)
     );
     dom.cl.toggle(listNode, 'searchMatch',
         qs$(listNode, ':scope > .listEntries > .listEntry.searchMatch') !== null
@@ -412,7 +414,7 @@ const updateListNode = listNode => {
     let isCached = false;
     let isObsolete = false;
     let writeTime = 0;
-    for ( const listLeaf of listLeaves ) {
+    for ( const listLeaf of checkedListLeaves ) {
         const listkey = listLeaf.dataset.key;
         const listDetails = listsetDetails.available[listkey];
         usedFilterCount += listDetails.off ? 0 : listDetails.entryUsedCount || 0;
@@ -422,8 +424,15 @@ const updateListNode = listNode => {
         isObsolete = isObsolete || dom.cl.has(listLeaf, 'obsolete');
         writeTime = Math.max(writeTime, assetCache.writeTime || 0);
     }
-    dom.cl.toggle(listNode, 'checked', listLeaves.length !== 0);
-    dom.prop(qs$(listNode, ':scope > .detailbar input'), 'checked', listLeaves.length !== 0);
+    dom.cl.toggle(listNode, 'checked', checkedListLeaves.length !== 0);
+    dom.cl.toggle(qs$(listNode, ':scope > .detailbar .checkbox'),
+        'partial',
+        checkedListLeaves.length !== allListLeaves.length
+    );
+    dom.prop(qs$(listNode, ':scope > .detailbar input'),
+        'checked',
+        checkedListLeaves.length !== 0
+    );
     dom.text(qs$(listNode, '.leafstats'),
         renderLeafStats(usedFilterCount, totalFilterCount)
     );
@@ -827,14 +836,14 @@ self.cloud.onPull = function fromCloudData(data, append) {
         selectedSet.delete(listkey);
     }
     if ( selectedSet.size !== 0 ) {
-        const textarea = qs$('#lists .liEntry[data-role="import"] textarea');
+        const textarea = qs$('#lists .listEntry[data-role="import"] textarea');
         const lines = append
             ? textarea.value.split(/[\n\r]+/)
             : [];
         lines.push(...selectedSet);
         if ( lines.length !== 0 ) { lines.push(''); }
         textarea.value = lines.join('\n');
-        dom.cl.toggle('#lists .liEntry[data-role="import"]', 'expanded', textarea.value !== '');
+        dom.cl.toggle('#lists .listEntry[data-role="import"]', 'expanded', textarea.value !== '');
     }
 
     renderWidgets();
