@@ -133,7 +133,9 @@ const hashFromPopupData = function(reset = false) {
     if ( reset ) {
         cachedPopupHash = hash;
     }
-    dom.cl.toggle(dom.body, 'needReload', hash !== cachedPopupHash);
+    dom.cl.toggle(dom.body, 'needReload',
+        hash !== cachedPopupHash || popupData.hasUnprocessedRequest === true
+    );
 };
 
 /******************************************************************************/
@@ -457,6 +459,75 @@ const reIP = /(\d|\])$/;
 
 /******************************************************************************/
 
+function filterFirewallRows() {
+    const firewallElem = qs$('#firewall');
+    const elems = qsa$('#firewall .filterExpressions span[data-expr]');
+    let not = false;
+    for ( const elem of elems ) {
+        const on = dom.cl.has(elem, 'on');
+        switch ( elem.dataset.expr ) {
+            case 'not':
+                not = on;
+                break;
+            case 'blocked':
+                dom.cl.toggle(firewallElem, 'showBlocked', !not && on);
+                dom.cl.toggle(firewallElem, 'hideBlocked', not && on);
+                break;
+            case 'allowed':
+                dom.cl.toggle(firewallElem, 'showAllowed', !not && on);
+                dom.cl.toggle(firewallElem, 'hideAllowed', not && on);
+                break;
+            case 'script':
+                dom.cl.toggle(firewallElem, 'show3pScript', !not && on);
+                dom.cl.toggle(firewallElem, 'hide3pScript', not && on);
+                break;
+            case 'frame':
+                dom.cl.toggle(firewallElem, 'show3pFrame', !not && on);
+                dom.cl.toggle(firewallElem, 'hide3pFrame', not && on);
+                break;
+            default:
+                break;
+        }
+    }
+}
+
+dom.on('#firewall .filterExpressions', 'click', 'span[data-expr]', ev => {
+    const target = ev.target;
+    dom.cl.toggle(target, 'on');
+    switch ( target.dataset.expr ) {
+        case 'blocked':
+            if ( dom.cl.has(target, 'on') === false ) { break; }
+            dom.cl.remove('#firewall .filterExpressions span[data-expr="allowed"]', 'on');
+            break;
+        case 'allowed':
+            if ( dom.cl.has(target, 'on') === false ) { break; }
+            dom.cl.remove('#firewall .filterExpressions span[data-expr="blocked"]', 'on');
+            break;
+    }
+    filterFirewallRows();
+    const elems = qsa$('#firewall .filterExpressions span[data-expr]');
+    const filters = Array.from(elems) .map(el => dom.cl.has(el, 'on') ? '1' : '0');
+    filters.unshift('00');
+    vAPI.localStorage.setItem('firewallFilters', filters.join(' '));
+});
+
+{
+    vAPI.localStorage.getItemAsync('firewallFilters').then(v => {
+        if ( v === null ) { return; }
+        const filters = v.split(' ');
+        if ( filters.shift() !== '00' ) { return; }
+        if ( filters.every(v => v === '0') ) { return; }
+        const elems = qsa$('#firewall .filterExpressions span[data-expr]');
+        for ( let i = 0; i < elems.length; i++ ) {
+            if ( filters[i] === '0' ) { continue; }
+            dom.cl.add(elems[i], 'on');
+        }
+        filterFirewallRows();
+    });
+}
+
+/******************************************************************************/
+
 const renderPrivacyExposure = function() {
     const allDomains = {};
     let allDomainCount = 0;
@@ -604,6 +675,9 @@ const renderPopup = function() {
         total ? Math.min(total, 99).toLocaleString() : ''
     );
 
+    // Unprocesseed request(s) warning
+    dom.cl.toggle(dom.root, 'warn', popupData.hasUnprocessedRequest === true);
+
     dom.cl.toggle(dom.html, 'colorBlind', popupData.colorBlindFriendly === true);
 
     setGlobalExpand(popupData.firewallPaneMinimized === false, true);
@@ -618,6 +692,18 @@ const renderPopup = function() {
 
 /******************************************************************************/
 
+dom.on('.dismiss', 'click', ( ) => {
+    messaging.send('popupPanel', {
+        what: 'dismissUnprocessedRequest',
+        tabId: popupData.tabId,
+    }).then(( ) => {
+        popupData.hasUnprocessedRequest = false;
+        dom.cl.remove(dom.root, 'warn');
+    });
+});
+
+/******************************************************************************/
+
 // https://github.com/gorhill/uBlock/issues/2889
 //   Use tooltip for ARIA purpose.
 
@@ -838,7 +924,7 @@ const gotoReport = function() {
         popupPanel[name] = !expected;
     }
     if ( hostnameToSortableTokenMap.size !== 0 ) {
-        const blockedDetails = {};
+        const network = {};
         const hostnames =
             Array.from(hostnameToSortableTokenMap.keys()).sort(hostnameCompare);
         for ( const hostname of hostnames ) {
@@ -846,20 +932,20 @@ const gotoReport = function() {
             const count = entry.counts.blocked.any;
             if ( count === 0 ) { continue; }
             const domain = entry.domain;
-            if ( blockedDetails[domain] === undefined ) {
-                blockedDetails[domain] = 0;
+            if ( network[domain] === undefined ) {
+                network[domain] = 0;
             }
-            blockedDetails[domain] += count;
+            network[domain] += count;
         }
-        if ( Object.keys(blockedDetails).length !== 0 ) {
-            popupPanel.blockedDetails = blockedDetails;
+        if ( Object.keys(network).length !== 0 ) {
+            popupPanel.network = network;
         }
     }
     messaging.send('popupPanel', {
         what: 'launchReporter',
         tabId: popupData.tabId,
-        pageURL: popupData.pageURL,
-        popupPanel: JSON.stringify(popupPanel),
+        pageURL: popupData.rawURL,
+        popupPanel,
     });
 
     vAPI.closePopup();
@@ -1066,10 +1152,22 @@ const setFirewallRuleHandler = function(ev) {
 /******************************************************************************/
 
 const reloadTab = function(bypassCache = false) {
+    // Preemptively clear the unprocessed-requests status since we know for sure
+    // the page is being reloaded in this code path.
+    if ( popupData.hasUnprocessedRequest === true )  {
+        messaging.send('popupPanel', {
+            what: 'dismissUnprocessedRequest',
+            tabId: popupData.tabId,
+        }).then(( ) => {
+            popupData.hasUnprocessedRequest = false;
+            dom.cl.remove(dom.root, 'warn');
+        });
+    }
+
     messaging.send('popupPanel', {
         what: 'reloadTab',
         tabId: popupData.tabId,
-        url: popupData.pageURL,
+        url: popupData.rawURL,
         select: vAPI.webextFlavor.soup.has('mobile'),
         bypassCache,
     });
@@ -1307,29 +1405,23 @@ const toggleHostnameSwitch = async function(ev) {
 // it and thus having to push it all the time unconditionally.
 
 const pollForContentChange = (( ) => {
-    let pollTimer;
-
     const pollCallback = async function() {
-        pollTimer = undefined;
         const response = await messaging.send('popupPanel', {
             what: 'hasPopupContentChanged',
             tabId: popupData.tabId,
             contentLastModified: popupData.contentLastModified,
         });
-        queryCallback(response);
-    };
-
-    const queryCallback = function(response) {
         if ( response ) {
-            getPopupData(popupData.tabId);
+            await getPopupData(popupData.tabId);
             return;
         }
         poll();
     };
 
+    const pollTimer = vAPI.defer.create(pollCallback);
+
     const poll = function() {
-        if ( pollTimer !== undefined ) { return; }
-        pollTimer = vAPI.setTimeout(pollCallback, 1500);
+        pollTimer.on(1500);
     };
 
     return poll;
@@ -1435,15 +1527,3 @@ dom.on('#revertRules', 'click', ( ) => { revertFirewallRules(); });
 dom.on('a[href]', 'click', gotoURL);
 
 /******************************************************************************/
-
-// Toggle emphasis of rows with[out] 3rd-party scripts/frames
-dom.on('#firewall > [data-type="3p-script"] .filter', 'click', ( ) => {
-    dom.cl.toggle('#firewall', 'show3pScript');
-});
-
-// Toggle visibility of rows with[out] 3rd-party frames
-dom.on('#firewall > [data-type="3p-frame"] .filter', 'click', ( ) => {
-    dom.cl.toggle('#firewall', 'show3pFrame');
-});
-
-/******************************************************************************/
